package com.ibm.ctg.ping;
/**
*   Class: RunningTotal
*   
*   Support pack for IBM CICS Transaction Gateway Version v5.0 to v7.2 
*
*   Version       : 1.3
*
*   Description   : Connects to a specified Gateway daemon, issues requests
*   and measures the time taken for operations to complete. This is useful for 
*   testing and debugging connectivity and performance.
*
*   See support pack documentation for details on how to configure and use this
*   sample. Also see the support pack for information on configuration necessary on
*   CICS.
*   
*   Licensed Materials - Property of IBM
*
*   Copyright IBM Corporation 2007,2008
*   All Rights Reserved
*
*   US Government Users Restricted Rights - Use,
*   duplication, or disclosure restricted by GSA
*   Schedule Contract with IBM Corporation
*
*
*   The following code is sample code created by IBM Corporation.  This
*   sample code is not part of any standard IBM product and is provided
*   to you solely for the purpose of assisting you in the development of
*   your applications.  The code is provided 'AS IS', without warranty
*   or condition of any kind.  IBM shall not be liable for any damages
*   arising out of your use of the sample code, even if IBM has been
*   advised of the possibility of such damages.
* 
*/

import java.text.DecimalFormat;

public class RunningTotal {

   /**
    * This class calculates average and maximum values.
    * It also formats the results as text.  
    */
   
   private static final String NOT_APPLICABLE = "*";
   private long minValue;
   private long maxValue;
   private long numRequests;
   private long totalValue;
   private int  precision;
   private long currentValue;
   DecimalFormat formatterForFloatingPoint=null;
   
   private long currentStart;
   private long currentEnd;
   private int failCount;
   
   /**
    * Set up an object for use. 
    * @param precision the minimum value that will be output. (If less than this
    * it will output '<x' where 'x' is the precision value).
    */
   RunningTotal(int precision) {
      minValue = Long.MAX_VALUE;
      maxValue = Long.MIN_VALUE;
      numRequests=0;
      totalValue=0;
      this.precision=precision;
      currentValue=0;
      
      formatterForFloatingPoint = new DecimalFormat();
      //Set the maximum number of digits after the decimal point to 1.
      //This keeps the average value displayed easy to understand by avoiding
      //outputting a level of precision that isn't needed.
      formatterForFloatingPoint.setMaximumFractionDigits(1);
   }
   
   void setStart() {
      currentStart=System.currentTimeMillis();
   }
   
   void setEnd() {
      currentEnd=System.currentTimeMillis();
   }
   
   void success() {
      this.addValue(currentEnd-currentStart);
   }
   
   /**
    * Add a value to the average and max/min calculations.
    * @param curVal the value to take account of.
    */
   private void addValue(long curVal) {
      totalValue+=curVal;
      numRequests++;
      
      if (curVal>maxValue) {
         maxValue=curVal;
      }
      if (curVal<minValue){
         minValue=curVal;        
      }
      currentValue=curVal;
      
   }

   /**
    * Get previous value added in the form of a formatted string
    * @return the current value.
    */
   public String getCurrentValStr() {
      return (currentValue>=precision? Long.toString(currentValue):("<"+precision));
   }
  
   /**
    * Get the maximum of all values that have been added.
    * @return the maximum value.
    */
   public String getMaxStr() {
      if (numRequests>0) {
         return (maxValue>=precision ? Long.toString(maxValue):("<"+precision)); 
      } else {
         return NOT_APPLICABLE;
      }
   }
     
  /**
   * Get the minimum of all values that have been added in the form of a formatted
   * String
   * @return minimum value
   */
  public String getMinStr(){
      if (numRequests>0) {
         return (minValue>=precision ? Long.toString(minValue):("<"+precision)); 
      } else {
         return NOT_APPLICABLE;
      }     
   }
     
   /**
    * Get the current average in the form of a formatted string.
    * @return the current average
    */
   public String getAverageStr() {
      if (numRequests>0) {
         double ave = (double)totalValue/(double)numRequests;
         String formattedAve = formatterForFloatingPoint.format(ave);   
         return (ave>=precision ? formattedAve:"<"+precision);
      } else {
         return NOT_APPLICABLE;
      }
   }

   /**
    * Get the number of requests that have been added. This total includes both
    * success and fail requests.
    * @return number of requests
    */
   public long getNumRequests() {
      return numRequests+failCount;
   }

   public void fail() {
      failCount++;   
   }
   
   public int getFailCount() {
      return failCount;
   }
}
